### xes constants

### holds the equivalent xes type names for the python type names
xes_typenames = {"str": 'string',
                 "datetime": 'date',
                 "Timestamp": 'date',
                 "int": 'int',
                 "float": 'float',
                 "bool": 'boolean',
                 "dict": 'list'
                }